Package: degradr
Type: Package
Title: Estimating Remaining Useful Life with Linear Mixed Effects
        Models
Description: Provides tools for estimating the Remaining Useful Life (RUL) of degrading systems using linear mixed-effects models and creating a health index. It supports both univariate and multivariate degradation signals. For multivariate inputs, the signals are merged into a univariate health index prior to modeling. Linear and exponential degradation trajectories are supported (the latter using a log transformation). Remaining Useful Life (RUL) distributions are estimated using Bayesian updating for new units, enabling on-site predictive maintenance. Based on the methodology of Liu and Huang (2016) <doi:10.1109/TASE.2014.2349733>.
Version: 1.0.2
Authors@R: c(person(given="Pedro Abraham", family="Montoya Calzada", role = c("aut", "cre", "cph"), 
            comment = c(ORCID = "0009-0002-3497-210X"),email = "pedroabraham.montoya@gmail.com"),
            person(given="Rogelio", family="Salinas Gutiérrez", role = c("aut","cph"),
                     email = "rsalinas@correo.uaa.mx", comment = c(ORCID = "0000-0002-1669-4460")),
            person(given="Silvia", family="Rodríguez-Narciso", role = c("aut","cph"),
                    comment = c(ORCID = "0000-0001-5429-5914")),
            person(given="Netzahualcóyotl", family="Castañeda-Leyva", role = c("aut","cph"),
                    comment = c(ORCID = "0000-0001-9414-3923")))
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.2.1
Imports: dplyr, nlme, quadprog, ggplot2
Maintainer: Pedro Abraham Montoya Calzada <pedroabraham.montoya@gmail.com>
Depends: R (>= 4.5)
LazyData: true
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-03-26 03:45:27 UTC; Abraham
Author: Pedro Abraham Montoya Calzada [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0002-3497-210X>),
  Rogelio Salinas Gutiérrez [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-1669-4460>),
  Silvia Rodríguez-Narciso [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-5429-5914>),
  Netzahualcóyotl Castañeda-Leyva [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-9414-3923>)
Repository: CRAN
Date/Publication: 2026-03-26 07:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 12:40:09 UTC; unix

export(fit_model)
export(predict_rul)
export(fit_healthindex)
export(plot_degradr)
export(compute_healthindex)
export(prul)
export(qrul)
importFrom(nlme, lme, fixed.effects, getVarCov)
import(dplyr)
import(quadprog)
import(ggplot2)
importFrom("stats", "AIC", "BIC", "as.formula", "logLik", "pnorm",
               "sigma", "var","coef", "cov", "lm", "residuals")

compute_healthindex <- function(model,data){
  w <- model$index$w
  phi <- model$index$phi
  type = model$model$type
  if(type == "exponential"){
    data_index <- log_transform(data = data, phi_list = phi)
  }else{
    data_index <- data
  }
  index <- data_index %>%
    mutate(x = as.vector(as.matrix(select(.,-unit,-t)) %*% w)) %>%
    select(unit,t,x)
  return(index)
}

model_nlme <- function(fixed_formula,random_formula,data){
  model <- nlme::lme(
    fixed = fixed_formula,
    random = random_formula,
    data = data
  )
  u0 <- fixed.effects(model)
  Sigma0 <- getVarCov(model)
  sigma2 <- sigma(model)^2
  return(list(model = model, u0 = u0, Sigma0 = Sigma0,
              sigma2 = sigma2))
}

fit_healthindex <- function(data, type = "exponential", method = "lm",
                            degree = 2, phi = NULL, r = 0.5){
  data$unit <- match(data$unit, unique(data$unit))
  index <- healthindex(data = data, type = type,
                       p = degree, r = r, phi = phi)
  w <- index$w
  phi <- index$phi
  if(type == "exponential"){
    data_index <- log_transform(data = data, phi_list = phi)
  }else{
    data_index <- data
  }
  data_model <- data_index %>%
    mutate(x = as.vector(as.matrix(select(.,-unit,-t)) %*% w)) %>%
    select(unit,t,x)
  model <- fit_model(data = data_model,type = type,
                     method = method, degree = degree, phi = NULL)
  structure(list(
    index = index,
    model = model
  ), class = "healthindex")
}

library(degradr)
library(dplyr)
test_that("fit_healthindex returns healthindex object", {
  data <- train_FD001 %>%
    select(unit,t,T24,T50,P30,
           Nf,Ps30,phi, NRf,
           BPR,htBleed,
           W31, W32) %>%
    mutate(across(c(P30,phi,W31,W32), ~ . * -1))
  hi <- fit_healthindex(data)
  expect_s3_class(hi, "healthindex")
  expect_true(all(c("index", "model") %in% names(hi)))
})

qRLD_random <- function(q, n, u1, sigma1,sigma2, ud,
                        vd, p, lower = 0, upper) {
  obj <- function(t) q - RLD_random(t = t, n = n, u1 = u1,
                                    sigma1 = sigma1,sigma2 = sigma2,
                                    ud = ud, vd = vd, p = p)
  uniroot(obj, lower = lower, upper = upper)$root
}
qRLD_random <- Vectorize(qRLD_random, vectorize.args = "q")

RLD_random <- function(t, n, u1, sigma1, sigma2, ud, vd, p) {
  gt <- g(t = t, n = n, u1 = u1, sigma1 = sigma1,
          sigma2 = sigma2, ud = ud, vd = vd, p = p)
  g0 <- g(0, n = n, u1 = u1, sigma1 = sigma1,
          sigma2 = sigma2, ud = ud, vd = vd, p = p)
  value <- (pnorm(gt) - pnorm(g0)) / (1 - pnorm(g0))
  return(value)
}

plot_degradr <- function(data,D = NULL){
  g <- ggplot(data = data) +
     geom_line(aes(x = t, y = x, group = unit))
  if(!is.null(D)){
    g <- g +
      geom_hline(yintercept = D, col = "red")
  }
  return(g)
}

g2 <- function(t, n, u1, sigma1, sigma2, p,D) {
  z <- (n + t)^(0:p)
  u_tilde <- sum(z * u1)
  sigma2_tilde <- ((t(z) %*% sigma1) %*% z ) + sigma2
  value <- (D - u_tilde)/sqrt(sigma2_tilde)
  return(value)
}

RLD <- function(t, n, u1, sigma1, sigma2, p,D){
  gt <- g2(t = t, n = n, u1 = u1, sigma1 = sigma1,
           sigma2 = sigma2, p = p, D = D)
  g0 <-  g2(t = 0, n = n, u1 = u1, sigma1 = sigma1,
            sigma2 = sigma2, p = p, D = D)
  value <- (pnorm(g0) - pnorm(gt))/(pnorm(g0))
  return(value)
}

qrul <- function(prob = 0.05,data, model, D = NULL, upper = NULL) {
  if (inherits(model, "healthindex")) {
    data_index <- log_transform(data = data, phi_list = model$index$phi)
    data <- data_index %>%
      mutate(x = as.vector(as.matrix(select(.,-unit,-t)) %*% model$index$w)) %>%
      select(unit,t,x)
    model <- model$model
  }


  if (!all(c("t", "x", "unit") %in% colnames(data))) {
    stop("Input data must have columns: 't', 'x', and 'unit'")
  }

  if (is.null(upper)) {
    upper <- model$upper
  }

  units_list <- split(data, data$unit)

  safe_qRLD <- function(q, ...) {
    tryCatch(
      qRLD(q, ...),
      error = function(e) {
        NA
      }
    )
  }

  safe_qRLD_random <- function(q, ...) {
    tryCatch(
      qRLD_random(q, ...),
      error = function(e) {
        NA
      }
    )
  }

  predict_one <- function(unit_data) {
    t <- unit_data$t
    x <- unit_data$x
    n <- max(t)
    post <- posterior(model = model, t = t, x = x)
    u1 <- post$u1
    sigma1 <- post$sigma1
    sigma2 <- model$sigma2
    ud <- model$ud
    vd <- model$vd
    p <- model$degree
    type <- model$type

    if (is.null(D)) {
      med <- safe_qRLD_random(prob, n = n, u1 = u1,
                              sigma1 = sigma1, sigma2 = sigma2,
                              ud = ud, vd = vd, p = p, upper = upper)
    } else {
      D_trans <- D
      if (type == "exponential") {
        phi <- model$phi
        D_trans <- log(D - phi)
      }
      med <- safe_qRLD(prob, n = n, u1 = u1,
                       sigma1 = sigma1, sigma2 = sigma2,
                       p = p, D = D_trans, upper = upper)
    }

    data.frame(
      unit = unique(unit_data$unit),
      RUL = med
    )
  }

  results <- lapply(units_list, predict_one)
  results_df <- dplyr::bind_rows(results)

  return(results_df)
}

model_lm <- function(fixed_formula, data) {
  unidades <- unique(data$unit)
  coef_list <- list()
  sigma2_list <- numeric(length(unidades))

  for (i in seq_along(unidades)) {
    df_u <- subset(data, unit == unidades[i])
    model_u <- lm(fixed_formula, data = df_u)

    # Guardar coeficientes
    coef_list[[i]] <- coef(model_u)

    # MSE residual para la unidad i
    sigma2_list[i] <- mean(residuals(model_u)^2)
  }

  coef_mat <- do.call(rbind, coef_list)

  u0 <- colMeans(coef_mat)
  Sigma0 <- cov(coef_mat)
  sigma2 <- mean(sigma2_list)  # Promedio de los MSE por unidad

  return(list(
    u0 = u0,
    Sigma0 = Sigma0,
    sigma2 = sigma2
  ))
}

log_transform <- function(data, phi_list, exclude = c("unit", "t")) {
  df_trans <- data
  for (var in names(phi_list)) {
    if (!var %in% names(data)) {
      stop(paste("Variable", var, "not found in the new dataset.."))
    }
    df_trans[[var]] <- log(data[[var]] - phi_list[[var]])
  }
  return(df_trans)
}

prul <- function(t, data, model, D = NULL){
  # Preprocesamiento si es un objeto healthindex (misma lógica que en qrul)
  if (inherits(model, "healthindex")) {
    data_index <- log_transform(data = data, phi_list = model$index$phi)
    data <- data_index %>%
      dplyr::mutate(x = as.vector(as.matrix(dplyr::select(., -unit, -t)) %*% model$index$w)) %>%
      dplyr::select(unit, t, x)
    model <- model$model
  }

  # Validación de columnas
  if (!all(c("t", "x", "unit") %in% colnames(data))) {
    stop("Input data must have columns: 't', 'x', and 'unit'")
  }

  # Split por unidad
  units_list <- split(data, data$unit)

  # Wrappers seguros (evitan que una unidad con error rompa todo)
  safe_RLD <- function(...) {
    tryCatch(RLD(...), error = function(e) NA_real_)
  }
  safe_RLD_random <- function(...) {
    tryCatch(RLD_random(...), error = function(e) NA_real_)
  }

  # Predicción por unidad
  predict_one <- function(unit_data) {
    tt <- unit_data$t
    x  <- unit_data$x
    n  <- max(tt)

    post <- posterior(model = model, t = tt, x = x)
    u1 <- post$u1
    sigma1 <- post$sigma1
    sigma2 <- model$sigma2
    ud <- model$ud
    vd <- model$vd
    p  <- model$degree

    # Tipo de modelo
    type <- if (inherits(model, "healthindex")) model$model$type else model$type

    # Selección de función según D y transformación si es exponencial
    if (is.null(D)) {
      val <- safe_RLD_random(t = t, n = n, u1 = u1, sigma1 = sigma1,
                             sigma2 = sigma2, ud = ud, vd = vd, p = p)
    } else {
      D_trans <- D
      if (type == "exponential") {
        phi <- model$phi
        D_trans <- log(D - phi)
      }
      val <- safe_RLD(t = t, n = n, u1 = u1, sigma1 = sigma1,
                      sigma2 = sigma2, p = p, D = D_trans)
    }

    data.frame(
      unit = unique(unit_data$unit),
      prob = round(val, 4)
    )
  }

  # Ejecutar por unidad y consolidar
  results <- lapply(units_list, predict_one)
  results_df <- dplyr::bind_rows(results)

  return(results_df)
}

posterior <- function(model,t,x){
  degree <- model$degree
  Sigma0 <- model$Sigma0
  sigma2 <- model$sigma2
  if(inherits(model, "healthindex")){
    type = model$model$type
  }else{
    type <- model$type
  }
  u0 <- model$u0
  if(type == "exponential"){
    phi <- model$phi
    df_posterior <- data.frame(t = t, x = x)
    df_posterior <- df_posterior %>%
      mutate(
        L = log(x - phi)
      )
    L <- df_posterior$L
    psi <- design_matrix(t,degree = degree)
    term1 <- solve((t(psi)%*%psi)/sigma2+solve(Sigma0))
    term2 <- (t(psi)%*%L)/sigma2 + solve(Sigma0)%*%u0
    u1 <- term1%*%term2
    sigma1 <- solve((t(psi)%*%psi)/sigma2 + solve(Sigma0))
  }else{
    df_posterior <- data.frame(t = t, x = x)
    L <- df_posterior$x
    psi <- design_matrix(t,degree = degree)
    term1 <- solve((t(psi)%*%psi)/sigma2+solve(Sigma0))
    term2 <- (t(psi)%*%L)/sigma2 + solve(Sigma0)%*%u0
    u1 <- term1%*%term2
    sigma1 <- solve((t(psi)%*%psi)/sigma2 + solve(Sigma0))
  }

  return(list(u1 = u1, sigma1 = sigma1))
}

compute_phi <- function(data, epsilon = 0.1) {
  vars <- setdiff(names(data), c("unit", "t"))
  phi_list <- lapply(data[vars], function(x) min(x) - epsilon * diff(range(x)))
  names(phi_list) <- vars
  return(phi_list)
}

qRLD <- function(q, n, u1, sigma1, sigma2,
                 p, D, lower = 0, upper) {
  obj <- function(t) q - RLD(t = t, n = n, u1 = u1,
                             sigma1 = sigma1,sigma2 = sigma2,
                             D = D, p = p)
  uniroot(obj, lower = lower, upper = upper)$root
}
qRLD <- Vectorize(qRLD, vectorize.args = "q")

healthindex <- function(data, type, p, r, phi){
  if(type == "exponential"){
    if(is.null(phi)){
      phi <- compute_phi(data = data)
    }
    df_trans <- log_transform(data = data, phi_list = phi)
    Y <- df_trans %>%
      group_by(unit) %>%
      slice_tail(n = 1) %>%
      ungroup() %>%
      select(-unit, -t)
    Y <- as.matrix(Y)
    m <- length(unique(data$unit))
    s <- dim(Y)[2]
    P_list <- list()
    B_list <- list()
    for(i in 1:m){
      data_unit <- df_trans %>%
        filter(unit == i)
      L <- data_unit %>%
        select(-unit,-t)
      L <- as.matrix(L)
      ti <- data_unit$t
      ni <- length(ti)
      c1 <- 0.9/ni
      ci <- c1 + (ti-1)*(2-2*c1*ni)/((ni-1)*ni)
      ci <- diag(ci)
      Psi <- design_matrix(t = ti, degree = p)
      H <- ci%*%Psi%*%solve(t(Psi)%*%(ci^2)%*%Psi)%*%t(Psi)%*%ci
      I <- diag(ni)
      Pi <- t(L)%*%(I - H)%*%ci%*%L
      P_list[[i]] <- Pi
      B_list[[i]] <- I
    }
    P <- do.call(cbind, P_list)
    B_list <- lapply(P_list, function(Pi) diag(ncol(Pi)))
    B <- do.call(rbind, B_list)
    O <- matrix(1, nrow = m, ncol = m)
    D <- (diag(m) - O/m)/(m-1)
    A <- (1 - r)*P%*%B/m + r*t(Y)%*%D%*%Y
    M <- diag(s)
  }else{
    #phi <- NULL
    df_trans <- data
    Y <- df_trans %>%
      group_by(unit) %>%
      slice_tail(n = 1) %>%
      ungroup() %>%
      select(-unit, -t)
    Y <- as.matrix(Y)
    m <- length(unique(data$unit))
    s <- dim(Y)[2]
    P_list <- list()
    B_list <- list()
    for(i in 1:m){
      data_unit <- df_trans %>%
        filter(unit == i)
      L <- data_unit %>%
        select(-unit,-t)
      L <- as.matrix(L)
      ti <- data_unit$t
      ni <- length(ti)
      c1 <- 0.9/ni
      ci <- c1 + (ti-1)*(2-2*c1*ni)/((ni-1)*ni)
      ci <- diag(ci)
      Psi <- design_matrix(t = ti, degree = p)
      H <- ci%*%Psi%*%solve(t(Psi)%*%(ci^2)%*%Psi)%*%t(Psi)%*%ci
      I <- diag(ni)
      Pi <- t(L)%*%(I - H)%*%ci%*%L
      P_list[[i]] <- Pi
      B_list[[i]] <- I
    }
    P <- do.call(cbind, P_list)
    B_list <- lapply(P_list, function(Pi) diag(ncol(Pi)))
    B <- do.call(rbind, B_list)
    O <- matrix(1, nrow = m, ncol = m)
    D <- (diag(m) - O/m)/(m-1)
    A <- (1 - r)*P%*%B/m + r*t(Y)%*%D%*%Y
    M <- diag(s)
  }

  Dmat <- A
  dvec <- rep(0, s)

  a1 <- t(M) %*% rep(1, s)
  Aeq <- matrix(a1, ncol = 1)

  Aineq <- -M
  bineq <- -rep(1, s)

  Amat <- cbind(Aeq, Aineq)
  bvec <- c(1, bineq)
  meq <- 1

  sol <- solve.QP(Dmat = Dmat, dvec = dvec,
                  Amat = Amat,
                  bvec = bvec,
                  meq = meq)
  w <- sol$solution
  structure(list(
    w = w,
    phi = phi,
    data = data
  ), class = "healthindex")
}

fit_model <- function(data, type = "exponential", method = "lm",
                      degree = 2, phi = NULL) {
  data$unit <- match(data$unit, unique(data$unit))
  if(type == "exponential"){
    if (is.null(phi)) {
      epsilon <- 0.1
      phi <- min(data$x) - epsilon * diff(range(data$x))
    }
    model_data <- data %>%
      mutate(
        L = log(x - phi)
      )

    data_last <- model_data %>%
      group_by(unit) %>%
      slice_tail(n = 1)
    time_terms <- paste0("I(t^", 1:degree, ")", collapse = " + ")
    fixed_formula <- as.formula(paste("L ~", time_terms))
    random_formula <- as.formula(paste("~", time_terms, "| unit"))

    model <- switch (method,
      "nlme" = model_nlme(
        fixed_formula = fixed_formula,
        random_formula = random_formula,
        data = model_data),
      "lm" = model_lm(
        fixed_formula = fixed_formula,
        data = model_data)
    )

    u0 <- model$u0
    Sigma0 <- model$Sigma0
    sigma2 <- model$sigma2
    ud <- mean(data_last$L)
    vd <- var(data_last$L)
    structure(list(
      model = model,
      u0 = u0,
      Sigma0 = Sigma0,
      sigma2 = sigma2,
      phi = phi,
      type = type,
      degree = degree,
      ud = ud,
      vd = vd,
      upper = max(data$t)
    ), class = "degradation_model")
  }else{
    data_last <- data %>%
      group_by(unit) %>%
      slice_tail(n = 1)
    time_terms <- paste0("I(t^", 1:degree, ")", collapse = " + ")
    fixed_formula <- as.formula(paste("x ~", time_terms))
    random_formula <- as.formula(paste("~", time_terms, "| unit"))

    model <- switch (method,
                     "nlme" = model_nlme(
                       fixed_formula = fixed_formula,
                       random_formula = random_formula,
                       data = data),
                     "lm" = model_lm(
                       fixed_formula = fixed_formula,
                       data = data)
    )

    u0 <- model$u0
    Sigma0 <- model$Sigma0
    sigma2 <- model$sigma2
    ud <- mean(data_last$x)
    vd <- var(data_last$x)
    structure(list(
      model = model$model,
      u0 = u0,
      Sigma0 = Sigma0,
      sigma2 = sigma2,
      type = type,
      degree = degree,
      ud = ud,
      vd = vd,
      upper = max(data$t)
    ), class = "degradation_model")
  }
}

library(degradr)

set.seed(123)

test_that("predict_rul returns expected columns", {
  colnames(filter_train) <- c("t", "x", "unit")
  colnames(filter_test)  <- c("t", "x", "unit", "RUL")
  model <- fit_model(data = filter_train, type = "exponential", degree = 1)
  pred <- predict_rul(data = filter_test, model = model, D = 600)
  expect_true(all(c("unit", "RUL") %in% colnames(pred)))
})

library(degradr)

set.seed(123)

test_that("fit_model returns degradation_model", {
  colnames(filter_train) <- c("t", "x", "unit")
  model <- fit_model(filter_train)
  expect_s3_class(model, "degradation_model")
  expect_true(all(c("u0", "Sigma0", "sigma2", "phi") %in% names(model)))
})

g <- function(t, n, u1, sigma1, sigma2, ud, vd, p) {
  z <- (n + t)^(0:p)
  u_tilde <- sum(z * u1)
  sigma2_tilde <- ((t(z) %*% sigma1) %*% z ) + sigma2
  g <- (u_tilde - ud) / sqrt(sigma2_tilde + vd)
  return(g)
}

\name{fit_model}
\alias{fit_model}
\title{
Fitting the Linear Mixed Effects Model
}
\description{
Fits a linear or exponential mixed-effects model of degree \(p\) for the degradation process.
}
\usage{
fit_model(data,
  type = "exponential",
  method = "lm",
  degree = 2,
  phi = NULL)
}
\arguments{
  \item{data}{A data frame with three columns: \code{t} for time, \code{x} for the degradation signal, and \code{unit} as the unit identifier. At least two distinct units are required.}

  \item{type}{Model type. Either \code{"linear"} or \code{"exponential"}. The exponential model applies a logarithmic transformation to \code{x - phi}. Default is \code{"exponential"}.}

  \item{method}{Estimation method. Either \code{"nlme"} to fit a nonlinear mixed-effects model using \code{nlme::lme()}, or \code{"lm"} to fit separate linear models per unit and estimate fixed and random effects from the set of coefficients. Default is \code{"lm"}.}

  \item{degree}{Degree of the polynomial model. Default is \code{2}. The fixed and random effects will include powers of time up to the specified degree.}

  \item{phi}{Initial degradation level for non-defective units.
  Used in the exponential model as a fixed offset to ensure that the logarithmic transformation is valid and interpretable.
  If \code{NULL}, it is automatically estimated as a value slightly below the minimum observed degradation level.
  Ignored when \code{type = "linear"}.}
}

\details{
This function fits a linear or exponential polynomial mixed-effects model of degree \code{p} to degradation data collected over time from multiple units. The model captures both fixed effects (population-level degradation trends) and random effects (unit-specific deviations).

The exponential model applies a logarithmic transformation with an offset parameter \code{phi}. The offset \code{phi} can be provided or automatically estimated from the data.

At least two distinct units are required to estimate random effects. The degree parameter controls the polynomial order for the time terms in both fixed and random effects.
}
\value{
  Returns a list with the estimated model and prior distributions.
}
\references{
{
Liu, K. and Huang, S. (2016). \emph{Integration of Data Fusion Methodology and Degradation Modeling Process to Improve Prognostics}. IEEE Transactions on Automation Science and Engineering, 13(1), 344--354.\doi{10.1109/TASE.2014.2349733}
}

}

\examples{
library(degradr)

# Load example data sets
data(filter_train)
data(filter_test)

# Show the original column names
colnames(filter_train)

# Rename the columns to match the expected format: t, x, unit
colnames(filter_train) <- c("t", "x", "unit")
colnames(filter_test)  <- c("t", "x", "unit", "RUL")

# Plot the training set
plot_degradr(data = filter_train, D = 600)

# Fit an exponential mixed-effects model of degree 1
model <- fit_model(data = filter_train, type = "exponential", degree = 1)

# Predict the remaining useful life (RUL) for the test units,
# assuming a fixed failure threshold D = 600
predict_rul(data = filter_test, model = model, D = 600)
}

\name{plot_degradr}
\alias{plot_degradr}
\title{
Plot Degradation Trajectories for Multiple Units
}
\description{
Generates a line plot of degradation signals over time for each unit, optionally overlaying a failure threshold line. This function is useful for visualizing degradation paths across multiple components or systems.
}
\usage{
plot_degradr(data, D = NULL)
}
\arguments{
  \item{data}{A data frame with three columns: \code{t} for time, \code{x} for the degradation signal, and \code{unit} as the unit identifier.}
  \item{D}{
  Optional numeric value indicating the failure threshold.
  }
}
\details{
The function is designed to work with degradation datasets where each row represents an observation of a unit at a particular time. The plot shows how the degradation variable \code{x} evolves over time \code{t} for each unit. This is especially useful for visual inspection before model fitting or threshold analysis.
}
\value{
Returns a \code{ggplot} object that can be further customized or directly printed.
}

\examples{
library(degradr)

# Load example data sets
data(filter_train)
data(filter_test)

# Show the original column names
colnames(filter_train)

# Rename the columns to match the expected format: t, x, unit
colnames(filter_train) <- c("t", "x", "unit")
plot_degradr(data = filter_train, D = 600)
}
\keyword{visualization}

\name{compute_healthindex}
\alias{compute_healthindex}
\title{
Constructing the Health Index on New Data Using Trained Weights
}
\description{
Given a fitted \code{"healthindex"} object, this function constructs the univariate health index for new multivariate sensor data by applying the stored projection (weights and offsets).
}
\usage{
compute_healthindex(model, data)
}
\arguments{
  \item{model}{An object of class \code{"healthindex"} returned by \code{\link{fit_healthindex}}.}

  \item{data}{A data frame with new sensor readings over time. Must include the columns \code{t} (time) and \code{unit} (unit identifier), plus the same set of degradation signals used at training. All degradation signals must follow an upward trend.}
}
\details{
This function applies the projection learned in the first stage of \code{\link{fit_healthindex}} to new data:
}
\value{
A data frame (tibble) with the columns:
  \item{unit}{Unit identifier.}
  \item{t}{Time index.}
  \item{x}{Constructed health index at each \code{(unit, t)}.}
}
\seealso{
\code{\link{fit_healthindex}} for learning the health index and mixed-effects model,
\code{\link{predict_rul}} for RUL prediction based on the fitted model.
}
\examples{
library(degradr)
library(dplyr)
# Load example data
data(train_FD001)
data(test_FD001)
data <- train_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

test <- test_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

# Fit a health index model (exponential trajectory of degree 2)
model <- fit_healthindex(data = data, type = "exponential",
                         degree = 2, r = 0.8)

# Construct the health index on new data using stored weights/offsets
hi_new <- compute_healthindex(model = model, data = test)
head(hi_new)
}
\keyword{models}

\name{test_FD001}
\alias{test_FD001}
\docType{data}
\title{
NASA Turbofan Engine Degradation Simulation Test Data (Subset FD001)
}
\description{
Truncated time series data from turbofan engine degradation simulations, generated using the C-MAPSS (Commercial Modular Aero-Propulsion System Simulation) model. This test dataset contains operational sensor data up to a point before failure, intended for validating prognostic algorithms that estimate Remaining Useful Life (RUL).
}
\format{
  A data frame with multiple observations (rows) on the following 24 variables (columns):
  \describe{
    \item{\code{unit}}{Engine unit number (identifier)}
    \item{\code{t}}{Time in cycles}
    \item{\code{T2}}{Total temperature at fan inlet (°R)}
    \item{\code{T24}}{Total temperature at LPC outlet (°R)}
    \item{\code{T30}}{Total temperature at HPC outlet (°R)}
    \item{\code{T50}}{Total temperature at LPT outlet (°R)}
    \item{\code{P2}}{Pressure at fan inlet (psia)}
    \item{\code{P15}}{Total pressure in bypass-duct (psia)}
    \item{\code{P30}}{Total pressure at HPC outlet (psia)}
    \item{\code{Nf}}{Physical fan speed (rpm)}
    \item{\code{Nc}}{Physical core speed (rpm)}
    \item{\code{epr}}{Engine pressure ratio (P50/P2)}
    \item{\code{Ps30}}{Static pressure at HPC outlet (psia)}
    \item{\code{phi}}{Ratio of fuel flow to Ps30 (pps/psi)}
    \item{\code{NRf}}{Corrected fan speed (rpm)}
    \item{\code{NRc}}{Corrected core speed (rpm)}
    \item{\code{BPR}}{Bypass Ratio}
    \item{\code{farB}}{Burner fuel-air ratio}
    \item{\code{htBleed}}{Bleed Enthalpy}
    \item{\code{Nf_dmd}}{Demanded fan speed (rpm)}
    \item{\code{PCNfR_dmd}}{Demanded corrected fan speed (rpm)}
    \item{\code{W31}}{HPT coolant bleed (lbm/s)}
    \item{\code{W32}}{LPT coolant bleed (lbm/s)}
  }
}
\details{
Key characteristics of this test dataset:
\itemize{
  \item Simulates progressive degradation in the High Pressure Compressor (HPC) module
  \item Time series are truncated prior to failure (true RUL values not included)
  \item Includes realistic measurement noise and unit-to-unit variability
  }
}
\references{
Saxena, A., Goebel, K., Simon, D., & Eklund, N. (2008). \emph{Damage propagation modeling for aircraft engine run-to-failure simulation}. In \emph{2008 International Conference on Prognostics and Health Management} (pp. 1--9). IEEE. \doi{10.1109/PHM.2008.4711414}
}
\examples{
data(test_FD001)
}
\keyword{datasets}

\name{predict_rul}
\alias{predict_rul}
\title{
Predicting Remaining Useful Life (RUL) from Degradation Signals
}
\description{
Estimates the Remaining Useful Life (RUL) for one or more partially observed degradation signals based on a previously fitted linear or exponential mixed-effects model.
}
\usage{
predict_rul(data, model, D = NULL, upper = NULL)
}
\arguments{
    \item{data}{A data frame with columns \code{t} (time), \code{x} (degradation measurement), and \code{unit} (unit identifier) or a data frame containing sensor readings over time: must include the columns \code{t} (time), \code{unit} (unit identifier), and multiple degradation signals. Multiple units can be passed simultaneously.}

  \item{model}{An object of class \code{"degradation_model"} produced by the \code{\link{fit_model}} function.}

  \item{D}{(Optional) Critical degradation threshold. If provided, it will be used to compute the RUL via a fixed-threshold model. If \code{NULL} (default), a random-threshold model will be used based on training data statistics.}

  \item{upper}{Optional upper bound for the search interval when solving for the quantiles of the RUL distribution. If \code{NULL}, the function will use the maximum observed time in the training data.}
}
\details{
This function applies Bayesian updating to compute the posterior distribution of the degradation model parameters for each unit, conditional on its observed signal. Then, it computes the Remaining Life Distribution (RLD) and returns the estimated Remaining Useful Life.

It supports both linear and exponential degradation models, matching the formulation used in \code{\link{fit_model}}. The posterior updating follows the methodology of Liu and Huang (2016).
}
\value{
A data frame with one row per unit and the following columns:
\describe{
  \item{unit}{Unit identifier.}
  \item{RUL}{Estimated RUL.}
}
}
\references{
Liu, K. and Huang, S. (2016). \emph{Integration of Data Fusion Methodology and Degradation Modeling Process to Improve Prognostics}. IEEE Transactions on Automation Science and Engineering, 13(1), 344--354. \doi{10.1109/TASE.2014.2349733}
}
\examples{
library(degradr)

# Load example data sets
data(filter_train)
data(filter_test)

# Show the original column names
colnames(filter_train)

# Rename the columns to match the expected format: t, x, unit
colnames(filter_train) <- c("t", "x", "unit")
colnames(filter_test)  <- c("t", "x", "unit", "RUL")

# Plot the training set
plot_degradr(data = filter_train, D = 600)

# Fit an exponential mixed-effects model of degree 1
model <- fit_model(data = filter_train, type = "exponential", degree = 1)

# Predict the remaining useful life (RUL) for the test units,
# assuming a fixed failure threshold D = 600
predict_rul(data = filter_test, model = model, D = 600)
}

\name{fit_healthindex}
\alias{fit_healthindex}
\title{
Fitting a Health Index Model from Multivariate Signals
}
\description{
Fits a health index–based degradation model by projecting multivariate sensor signals into a univariate health index and modeling its evolution using a linear mixed-effects model.
}
\usage{
fit_healthindex(data,
  type = "exponential",
  method = "lm",
  degree = 2,
  phi = NULL,
  r = 0.5)
}
\arguments{
  \item{data}{A data frame containing sensor readings over time. Must include the columns \code{t} (time), \code{unit} (unit identifier), and multiple degradation signals. All degradation signals must have an upward trend.}
  \item{type}{Model type. Either \code{"linear"} or \code{"exponential"}. The exponential model applies a logarithmic transformation to \code{x - phi}. Default is \code{"exponential"}.}

  \item{method}{Estimation method. Either \code{"nlme"} to fit a nonlinear mixed-effects model using \code{nlme::lme()}, or \code{"lm"} to fit separate linear models per unit and estimate fixed and random effects from the set of coefficients. Default is \code{"lm"}.}

  \item{degree}{Degree of the polynomial model. Default is \code{2}. The fixed and random effects will include powers of time up to the specified degree.}

  \item{phi}{Initial degradation level for non-defective units.
  Used in the exponential model as a fixed offset to ensure that the logarithmic transformation is valid and interpretable.
  If \code{NULL}, it is automatically estimated as a value slightly below the minimum observed degradation level.
  Ignored when \code{type = "linear"}.}

  \item{r}{parameter that controls the relative importance of the threshold variance and the weighted residual sum of squares in the index-fitted degradation model.}
}
\details{
This function implements a two-stage modeling strategy. In the first stage, a univariate health index is constructed as a weighted linear combination of the input signals, using correlation-based shrinkage. In the second stage, the resulting health index is modeled over time with a linear mixed-effects model (on the log scale for exponential models).

The exponential model uses a log transformation of \code{x - phi}, where \code{phi} ensures positivity and interpretability. The \code{phi} parameter can be supplied by the user or estimated automatically.

The resulting object stores both the projection (health index definition) and the fitted model used for RUL prediction.
}
\value{
Returns an object of class \code{"healthindex"}, which contains:
  \item{index}{A list with components: weights \code{w}, offset \code{phi}, and raw projections.}
  \item{model}{A fitted mixed-effects model of the health index over time.}
}
\references{
Liu, K. and Huang, S. (2016). \emph{Integration of Data Fusion Methodology and Degradation Modeling Process to Improve Prognostics}. IEEE Transactions on Automation Science and Engineering, 13(1), 344--354.\doi{10.1109/TASE.2014.2349733}
}
\examples{
library(degradr)
library(dplyr)
# Load example data
data(train_FD001)
data(test_FD001)
data <- train_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

test <- test_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

# Fit a health index model (exponential trajectory of degree 2)
model <- fit_healthindex(data = data, type = "exponential",
                         degree = 2, r = 0.8)
rul <- predict_rul(data = test, model = model)
head(rul)
}
\keyword{models}

\name{RUL_FD001}
\alias{RUL_FD001}
\docType{data}
\title{
Remaining Useful Life (RUL) Ground Truth for FD001 Dataset
}
\description{
Ground truth values of the Remaining Useful Life (RUL) for each engine unit in the FD001 subset of the C-MAPSS dataset. These values correspond to the last observed cycle in the test set and are used for evaluation purposes in prognostics models.
}
\usage{data("RUL_FD001")}
\format{
  A data frame with 100 observations on the following variable:
  \describe{
    \item{\code{RUL}}{Numeric vector indicating the true Remaining Useful Life (in cycles) for each unit in the test set.}
  }
}
\details{
This dataset is part of the C-MAPSS (Commercial Modular Aero-Propulsion System Simulation) benchmark and is used as ground truth for performance evaluation of predictive maintenance models, particularly for models estimating Remaining Useful Life (RUL) under the FD001 operating condition scenario.
}
\references{
Saxena, A., Goebel, K., Simon, D., & Eklund, N. (2008). \emph{Damage propagation modeling for aircraft engine run-to-failure simulation}. In \emph{2008 International Conference on Prognostics and Health Management} (pp. 1--9). IEEE. \doi{10.1109/PHM.2008.4711414}
}
\examples{
data(RUL_FD001)
}
\keyword{datasets}

\name{train_FD001}
\alias{train_FD001}
\docType{data}
\title{
NASA Turbofan Engine Degradation Simulation Data (FD001)
}
\description{
Run-to-failure simulation data for aircraft turbofan engines generated using C-MAPSS (Commercial Modular Aero-Propulsion System Simulation). This dataset represents engine degradation in the High Pressure Compressor (HPC) module under varying operational conditions.
}
\usage{data("train_FD001")}
\format{
  A data frame with multiple observations (rows) on the following 24 variables (columns):
  \describe{
    \item{\code{unit}}{Engine unit number (identifier)}
    \item{\code{t}}{Time in cycles}
    \item{\code{T2}}{Total temperature at fan inlet (°R)}
    \item{\code{T24}}{Total temperature at LPC outlet (°R)}
    \item{\code{T30}}{Total temperature at HPC outlet (°R)}
    \item{\code{T50}}{Total temperature at LPT outlet (°R)}
    \item{\code{P2}}{Pressure at fan inlet (psia)}
    \item{\code{P15}}{Total pressure in bypass-duct (psia)}
    \item{\code{P30}}{Total pressure at HPC outlet (psia)}
    \item{\code{Nf}}{Physical fan speed (rpm)}
    \item{\code{Nc}}{Physical core speed (rpm)}
    \item{\code{epr}}{Engine pressure ratio (P50/P2)}
    \item{\code{Ps30}}{Static pressure at HPC outlet (psia)}
    \item{\code{phi}}{Ratio of fuel flow to Ps30 (pps/psi)}
    \item{\code{NRf}}{Corrected fan speed (rpm)}
    \item{\code{NRc}}{Corrected core speed (rpm)}
    \item{\code{BPR}}{Bypass Ratio}
    \item{\code{farB}}{Burner fuel-air ratio}
    \item{\code{htBleed}}{Bleed Enthalpy}
    \item{\code{Nf_dmd}}{Demanded fan speed (rpm)}
    \item{\code{PCNfR_dmd}}{Demanded corrected fan speed (rpm)}
    \item{\code{W31}}{HPT coolant bleed (lbm/s)}
    \item{\code{W32}}{LPT coolant bleed (lbm/s)}
  }
}
\details{
The data was generated for the Prognostics and Health Management (PHM) 2008 data challenge. Each engine unit starts with some initial wear and progresses to failure as efficiency and flow parameters degrade exponentially. The failure criterion is when the health index (calculated from stall margins and EGT) reaches zero. The dataset includes sensor measurements taken at cruise conditions.
}
\references{
Saxena, A., Goebel, K., Simon, D., & Eklund, N. (2008). \emph{Damage propagation modeling for aircraft engine run-to-failure simulation}. In \emph{2008 International Conference on Prognostics and Health Management} (pp. 1--9). IEEE. \doi{10.1109/PHM.2008.4711414}
}

\examples{
data(train_FD001)
}
\keyword{datasets}

\name{prul}
\alias{prul}
\title{
Probability of Remaining Useful Life (RUL) Falling Within a Time Horizon
}
\description{
Evaluates the cumulative probability that the Remaining Useful Life (RUL) of a unit is less than or equal to a specified time \code{t}. The computation is based on a fitted degradation model and the observed degradation signal for the unit.
}
\usage{
prul(t, data, model, D = NULL)
}
\arguments{
  \item{t}{Time at which to evaluate the RUL cumulative distribution function.}
  \item{data}{A data frame with columns \code{t} (time), \code{x} (degradation measurement), and \code{unit} (unit identifier) or a data frame containing sensor readings over time: must include the columns \code{t} (time), \code{unit} (unit identifier), and multiple degradation signals. Multiple units can be passed simultaneously.}
  \item{model}{An object of class \code{"degradation_model"} produced by \code{\link{fit_model}} or a \code{"healthindex"} object returned by \code{\link{fit_healthindex}}.}
  \item{D}{Optional critical degradation threshold. If provided, a fixed-threshold model is used; otherwise a random-threshold model is assumed. For exponential models the threshold is automatically transformed to the log scale.}
}
\details{
For a fixed threshold model (\code{D} supplied), the function computes the Remaining Life Distribution (RLD) using the specified failure threshold. If \code{D} is \code{NULL}, the distribution is computed under a random-threshold formulation based on the training data.
}
\value{
Numeric value between 0 and 1 giving \eqn{P(\mathrm{RUL} \le t)}.
}
\seealso{\code{\link{qrul}}, \code{\link{predict_rul}}}
\examples{
library(degradr)
library(dplyr)
# Load example data
data(train_FD001)
data(test_FD001)
data <- train_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

test <- test_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

# Fit a health index model (exponential trajectory of degree 2)
model <- fit_healthindex(data = data, type = "exponential",
                         degree = 2, r = 0.8)
#Probability that the run length will be less than or equal to 86 cycles
head(prul(t = 86, data = test, model = model))

}
\keyword{models}

\name{qrul}
\alias{qrul}
\title{
Quantiles of the Remaining Useful Life (RUL) Distribution
}
\description{
Returns quantiles of the Remaining Life Distribution for one or more units based on their observed degradation signals and a fitted model.
}
\usage{
qrul(prob = 0.05, data, model, D = NULL, upper = NULL)
}
\arguments{
  \item{prob}{Probability at which to evaluate the quantile of the RUL distribution.}
  \item{data}{A data frame with columns \code{t} (time), \code{x} (degradation measurement), and \code{unit} (unit identifier) or a data frame containing sensor readings over time: must include the columns \code{t} (time), \code{unit} (unit identifier), and multiple degradation signals. Multiple units can be passed simultaneously.}
  \item{model}{An object of class \code{"degradation_model"} returned by \code{\link{fit_model}} or a \code{"healthindex"} object from \code{\link{fit_healthindex}}.}
  \item{D}{Optional critical degradation threshold. If \code{NULL}, a random-threshold model is used; otherwise the provided threshold is used. For exponential models the threshold is internally log-transformed.}
  \item{upper}{Optional upper bound for the numerical search when computing quantiles. If \code{NULL}, the maximum time observed during training is used.}
}
\details{
For each unit in \code{data} the function computes the posterior distribution of the model parameters and evaluates the specified quantile of the Remaining Life Distribution. Units that result in computational errors return \code{NA}.
}
\value{
A data frame with one row per unit containing:
\item{unit}{Unit identifier.}
\item{RUL}{The requested quantile of the RUL distribution.}
}
\seealso{\code{\link{prul}}, \code{\link{predict_rul}}}
\examples{

library(degradr)
library(dplyr)
# Load example data
data(train_FD001)
data(test_FD001)
data <- train_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

test <- test_FD001 \%>\%
  select(unit,t,T24,T50,P30,
         Nf,Ps30,phi, NRf,
         BPR,htBleed,
         W31, W32) \%>\%
  mutate(across(c(P30,phi,W31,W32), ~ . * -1))

# Fit a health index model (exponential trajectory of degree 2)
model <- fit_healthindex(data = data, type = "exponential",
                         degree = 2, r = 0.8)
head(qrul(prob = 0.05, data = test, model = model))
}
\keyword{models}

\name{filter_train}
\docType{data}
\alias{filter_train}
\title{Training Data for Filter Degradation and RUL Prediction}
\description{
Training data from a real-world degradation process involving the clogging of gas filters. The dataset includes right-censored lifetimes due to a preventive maintenance policy. Failure is defined when pressure differential exceeds 600 Pa.
}
\usage{
data(filter_train)
}
\format{
A data frame with 49 degradation trajectories. Variables include:
\describe{
  \item{Time}{Time}
  \item{Differential_pressure}{Differential pressure across the filter in Pascal (Pa).}
  \item{Data_No}{Unique identifier for each filter test unit. Integer from 1 to 50.}
}
}
\source{
Adapted from: \cr
Hagmeyer, S., Mauthe, F., & Zeiler, P. (2021). \emph{Creation of Publicly Available Data Sets for Prognostics and Diagnostics Addressing Data Scenarios Relevant to Industrial Applications}. International Journal of Prognostics and Health Management, 12(2). \doi{10.36001/ijphm.2021.v12i2.3087}
}
\references{
See full description in the dataset repository on Kaggle by "Prognostics @ HSE".
}
